package memory

import (
	"context"
	"errors"
	"slices"

	"go.opentelemetry.io/otel/attribute"
	"go.opentelemetry.io/otel/codes"

	pb "gitea.libretechconsulting.com/rmcguire/ambient-local-exporter/api/v1alpha1/weather"
	"gitea.libretechconsulting.com/rmcguire/ambient-local-exporter/pkg/util"
	"gitea.libretechconsulting.com/rmcguire/ambient-local-exporter/pkg/weather"
)

func (r *MemoryRecorder) Get(ctx context.Context, req *pb.GetWeatherRequest) (
	[]*weather.WeatherUpdate, error,
) {
	ctx, span := r.tracer.Start(ctx, "memoryRecorder.Get")
	defer span.End()

	r.RLock()
	defer r.RUnlock()

	span.AddEvent("acquired lock on recorder cache")

	if r.count() == 0 {
		err := errors.New("no recorded updates to get")
		span.RecordError(err)
		return nil, err
	} else if r.count() <= int(*req.Limit) {
		span.RecordError(errors.New("requested more updates than recorded"))
	}

	updates := r.getUpdatesFromReq(req)
	span.AddEvent("request limit/opts applied to updates")

	span.SetAttributes(attribute.Int("retrieved", len(updates)))
	span.SetStatus(codes.Ok, "")

	return updates, nil
}

func (r *MemoryRecorder) getUpdatesFromReq(req *pb.GetWeatherRequest) []*weather.WeatherUpdate {
	if req.Opts == nil {
		return limitUpdates(r.updates, int(req.GetLimit()))
	}

	return r.applyOptsToUpdates(r.updates, int(req.GetLimit()), req.Opts)
}

func (r *MemoryRecorder) applyOptsToUpdates(updates []*weather.WeatherUpdate, limit int, opts *pb.GetWeatherOpts) []*weather.WeatherUpdate {
	if opts == nil {
		return updates
	} else if opts.StationName == nil && opts.StationType == nil {
		return updates
	}

	filtered := make([]*weather.WeatherUpdate, 0, limit)

	for i := len(updates) - 1; i >= 0; i-- {
		update := updates[i]
		match := true

		if opts.GetStationName() != "" {
			if update.GetStationName() != opts.GetStationName() {
				match = false
			}
		}
		if opts.GetStationType() != "" {
			if util.DerefStr(update.StationType) != opts.GetStationType() {
				match = false
			}
		}

		if match {
			filtered = append(filtered, update)
			if len(filtered) >= limit {
				return filtered
			}
		}
	}

	return slices.Clip(filtered)
}

func (r *MemoryRecorder) Count(ctx context.Context) int {
	_, span := r.tracer.Start(ctx, "countWeatherRecorder")
	defer span.End()

	count := r.count()

	span.SetAttributes(attribute.Int("count", count))
	span.SetStatus(codes.Ok, "")

	return count
}

func (r *MemoryRecorder) count() int {
	return len(r.updates)
}

func limitUpdates(updates []*weather.WeatherUpdate, limit int) []*weather.WeatherUpdate {
	if len(updates) > limit {
		return updates[len(updates)-limit:]
	}
	return updates
}
